#######################################################################################################
# File-Name:       BeggAndGreeneSampleSizeCalculation.R           
# Date:            7.18.2014                                
# Author:          (Nigel) Benjamin Neely (nigelneely@gmail.com)
# Purpose:         To calculate the sample size needed after accounting for symptom classes. This is a 
#                  modified function based on the paper "Assessment of Diagnostic Tests When Disease 
#                  Verification is Subject to Selection Bias" - Colin B. Begg and Robert A. Greenes
#                  Biometrics, Vol. 39, No. 1 (Mar., 1983), pp. 207-215
# Data Used:       N/A
# Packages Used:   plyr,reshape2
#######################################################################################################
# NOTE: If you are running this in the R console you must use the 'setwd' command to set the 
# working directory for the console to whereever you have saved this file prior to running.
# Otherwise you will see errors when loading data or saving figures!
#######################################################################################################
BeggAndGreeneSampleSizeCalculation <- function(phenotype.names,                 #List of yes/no variables to derive sample sizes around
                                               symptom_class,                   #Variable in the data set that contains "symptom classes" mutually exclusive grouping
                                               Freq,                            #Variable in the data set that contains Frequency by symptom class
                                               vbound.tpf=NULL,                 #This is the variance bound in canonical form (not the logit) around TPF
                                               assumed.tpf=NULL,                #Assumed value of True Positive Fraction (TPF)  in the population
                                               assumed.ppv.symptom.Rplus=NULL,  #Assumed value of PPV in each symptom class     in the sample (test positive)
                                               assumed.ppv.symptom.Rminus=NULL, #Assumed value of PPV in each symptom class     in the sample (test negative)
                                               vbound.fpf=NULL,                 #This is the variance bound in canonical form (not the logit) around FPF
                                               assumed.fpf=NULL,                #Assumed value of False Positive Fraction (FPF) in the population
                                               assumed.npv.symptom.Rplus=NULL,  #Assumed value of NPV in each symptom class     in the sample (test positive)
                                               assumed.npv.symptom.Rminus=NULL, #Assumed value of NPV in each symptom class     in the sample (test negative)
                                               dat) {                           #Data that contains phenotypes and symptom_class variable
    suppressMessages(require(plyr))
    suppressMessages(require(reshape2))
    #Standardized the input that we'll feed into this program
    #Inputs to the function
    dat <- dat[order(eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))),]
    #get actual data
    sc <- eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))
    ################################################################################################################################
    #Please note the following designation for letters used throughout this program:
    #   R,       value of diagnostic test result   (i.e. Authoritative Source)
    #   D,       disease status                    (i.e. Gold Standard)
    #   V,       selection variable               
    #     V+:    case selected for definitive disease verification
    #     V-:    case NOT selected for definitive disease verification
    #   X,       concomitant information (vector)
    #Using these variables we will determine the sample size needed to determine certain sensitivity/specificity defintions
    ################################################################################################################################
    #Gather some inital information about incoming information and instantiate the N.symptom dataset
    #This data set will be used to generate all information regarding N++, N+1, n+1, information as seen in the theoretical statistical appendix
    n.test                <- length(phenotype.names)
    n.symptoms            <- length(unique(eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))))
    #Not elegant, but works. Loop through to generate a data set that will contain information that will allow us to calculate sample size
    #based on two statistics: (1) Sensisitivity and/or (2)Specificity
    for (i in 1:n.test) {
      if (i==1) {
        N.symptom         <- data.frame(symptom_class=numeric(),
                                        R=numeric(),
                                        total=numeric(),
                                        authSource=character())
      }
      templateTemp <- data.frame(expand.grid(symptom_class=unique(sc),
                                             R=unique(eval(parse(text = paste("dat$", deparse(phenotype.names[i]), sep=""))))))
      temp                <- data.frame(ddply(dat,.(sc,
                                                    eval(parse(text = paste("dat$", deparse(phenotype.names[i]), sep="")))),
                                        summarise,total=sum(Freq)))
      colnames(temp)      <- c("symptom_class","R","total")
      final               <- merge(temp,templateTemp,by=c("symptom_class","R"),all.y=TRUE)
      final$total         <- ifelse(is.na(final$total),0,final$total)
      final$authSource    <- rep(phenotype.names[i],length(final$total))
      N.symptom           <- data.frame(rbind(N.symptom,final))
      rm(temp,final,templateTemp)
    }
    #We'll need to parse this information out, when $=1
    N.symptom.plus        <- subset(N.symptom,R==1)
    N.symptom.minus       <- subset(N.symptom,R==0)
    
    N.symptom.Rplus       <- dcast(melt(N.symptom.plus,id.vars=c("symptom_class","authSource"),measure.vars="total"),authSource~symptom_class)
    rownames(N.symptom.Rplus) <- N.symptom.Rplus$authSource
    N.symptom.Rplus.mat   <- data.matrix(N.symptom.Rplus[2:ncol(N.symptom.Rplus)],rownames.force=TRUE)
    N.symptom.Rminus      <- dcast(melt(N.symptom.minus,id.vars=c("symptom_class","authSource"),measure.vars="total"),authSource~symptom_class)
    rownames(N.symptom.Rminus) <- N.symptom.Rminus$authSource
    N.symptom.Rminus.mat  <- data.matrix(N.symptom.Rminus[2:ncol(N.symptom.Rminus)],rownames.force=TRUE)
    N.symptom.plus.plus   <- as.numeric(by(eval(parse(text = paste("dat$", deparse(Freq), sep=""))),
                                           eval(parse(text = paste("dat$", deparse(symptom_class), sep=""))), 
                                           sum))
    names(N.symptom.plus.plus) <- colnames(N.symptom.Rminus.mat)
    rm(N.symptom.plus,N.symptom.minus,N.symptom.Rplus,N.symptom.Rminus)
    
    beta                  <- N.symptom.plus.plus/sum(N.symptom.plus.plus)
    sensitivity           <- list()
    specificity           <- list()
    ####################################################################
    #Sensitivity Calculation
    ####################################################################
    if ((!is.null(assumed.tpf)) & (!is.null(assumed.ppv.symptom.Rplus)) & (!is.null(assumed.ppv.symptom.Rminus))) {
    vboundonlogit.tpf     <- vbound.tpf*((1/assumed.tpf)+(1/(1-assumed.tpf)))**2
    k.tpf                 <- vboundonlogit.tpf
    #Before we proceed, we need to make sure that the ppv/npv vector names match symptom classes fed
    assumed.ppv.symptom.Rplus.mat   <- unlist(assumed.ppv.symptom.Rplus[order(names(assumed.ppv.symptom.Rplus))])
    assumed.ppv.symptom.Rminus.mat  <- unlist(assumed.ppv.symptom.Rminus[order(names(assumed.ppv.symptom.Rminus))])
    if (sum(names(assumed.ppv.symptom.Rplus.mat) == sort(unique(as.character(sc))))!=length(unique(as.character(sc)))){
        stop('assumed.ppv.symptom.Rplus names do not match symptom class category names')
      }
    if (sum(names(assumed.ppv.symptom.Rminus.mat) == sort(unique(as.character(sc))))!=length(unique(as.character(sc)))){
      stop('assumed.ppv.symptom.Rminus names do not match symptom class category names')
    }
    weight.mat            <- array(NA, c(n.test, n.symptoms))
    sampsi.mat            <- rep(NA, n.test)
    theta                 <- array(NA, c(n.test, n.symptoms))
    alpha1                <- rep(NA, n.test)
    alpha2                <- rep(NA, n.test)
    A                     <- array(NA, c(n.test, n.symptoms))
    B                     <- array(NA, c(n.test, n.symptoms))
    
    for(i in 1:n.test) {
        theta[i,]         <- N.symptom.Rplus.mat[i,]/N.symptom.plus.plus
        alpha1[i]         <- 1/sum(beta*theta[i,]*assumed.ppv.symptom.Rplus.mat)
        alpha2[i]         <- 1/sum(beta*(1-theta[i,])*assumed.ppv.symptom.Rminus.mat)
        A[i,]                 <- (alpha1[i] * beta * theta[i,])^2 * (N.symptom.plus.plus/ifelse(N.symptom.Rplus.mat[i,]==0,10^1000000,N.symptom.Rplus.mat[i,])) *
                                                          assumed.ppv.symptom.Rplus.mat * (1-assumed.ppv.symptom.Rplus.mat)
        test              <- rep(NA, NROW(N.symptom.plus.plus))
        for(ii in 1:NROW(N.symptom.plus.plus)) {
            test[ii]      <- ifelse( (N.symptom.plus.plus[ii] - N.symptom.Rplus.mat[i,][ii])==0,
                                          0,
                                          N.symptom.plus.plus[ii] / (N.symptom.plus.plus[ii] - N.symptom.Rplus.mat[i,][ii]))
        }
        B[i,]              <- (alpha2[i] * beta * (1-theta[i,]))^2 * (test) * assumed.ppv.symptom.Rminus.mat * (1-assumed.ppv.symptom.Rminus.mat);
        weight.mat[i,]    <- sqrt(A[i,]+B[i,])/sum(sqrt(A[i,]+B[i,]))
        sampsi.mat[i]     <- 1 / k.tpf * sum(sqrt(A[i,] + B[i,])) ^ 2
    }
    #Here is the final output for 1 particular combination of 
    sampsi.symptom.mat  <- ceiling(round(weight.mat,3)*ceiling(sampsi.mat))
    out.mat             <- (cbind(round(weight.mat,3), sampsi.symptom.mat, apply(sampsi.symptom.mat,1,sum)))
    colnames(out.mat)   <- c(paste(colnames(N.symptom.Rplus.mat),": W",sep=""),paste(colnames(N.symptom.Rplus.mat),": N",sep=""),"Total")
    rownames(out.mat)   <- rownames(N.symptom.Rplus.mat)
    
    #Adding the following information to communicate to clinicians
    se <- sqrt(vboundonlogit.tpf)
    lclLOGIT <- log(assumed.tpf/(1-assumed.tpf))-qnorm(0.975)*sqrt(vboundonlogit.tpf)
    uclLOGIT <- log(assumed.tpf/(1-assumed.tpf))+qnorm(0.975)*sqrt(vboundonlogit.tpf)
    lcl      <- (exp(lclLOGIT)/(1+exp(lclLOGIT)))
    ucl      <- (exp(uclLOGIT)/(1+exp(uclLOGIT)))
    
    sensitivity <- list(BETAS  = beta,
                        THETA  = theta,
                        ALPHA1 = alpha1,
                        ALPHA2 = alpha2,
                        A      = A,
                        B      = B,
                        RESULT = out.mat,
                        assumed.tpf = assumed.tpf,
                        lcl.assumed.tpf = lcl,
                        ucl.assumed.tpf = ucl,
                        assumptions = list(
                          vbound.tpf        = match.call()[[5]],
                          tpf               = match.call()[[6]],
                          ppv.symptom.Rplus = match.call()[[7]],
                          ppv.symptom.Rminus= match.call()[[8]])
                        )
    }
    ####################################################################
    #Specificity Calculation
    ####################################################################
    if ((!is.null(assumed.fpf)) & (!is.null(assumed.npv.symptom.Rplus)) & (!is.null(assumed.npv.symptom.Rminus))) {
    vboundonlogit.fpf     <- vbound.fpf*((1/assumed.fpf)+(1/(1-assumed.fpf)))**2
    k.fpf                 <- vboundonlogit.fpf
    assumed.npv.symptom.Rplus.mat   <- unlist(assumed.npv.symptom.Rplus[order(names(assumed.npv.symptom.Rplus))])
    assumed.npv.symptom.Rminus.mat  <- unlist(assumed.npv.symptom.Rminus[order(names(assumed.npv.symptom.Rminus))])
    if (sum(names(assumed.npv.symptom.Rplus.mat) == sort(unique(as.character(sc))))!=length(unique(as.character(sc)))){
      stop('assumed.npv.symptom.Rplus names do not match symptom class category names')
    }
    if (sum(names(assumed.npv.symptom.Rminus.mat) == sort(unique(as.character(sc))))!=length(unique(as.character(sc)))){
      stop('assumed.npv.symptom.Rminus names do not match symptom class category names')
    }
    weight.mat            <- array(NA, c(n.test, n.symptoms))
    sampsi.mat            <- rep(NA, n.test)
    theta                 <- array(NA, c(n.test, n.symptoms))
    alpha1                <- rep(NA, n.test)
    alpha2                <- rep(NA, n.test)
    A                     <- array(NA, c(n.test, n.symptoms))
    B                     <- array(NA, c(n.test, n.symptoms))
    for(i in 1:n.test) {
      theta[i,]               <- N.symptom.Rminus.mat[i,]/N.symptom.plus.plus 
      alpha1[i]           <- 1/sum(beta*theta[i,]*assumed.npv.symptom.Rminus.mat)
      alpha2[i]           <- 1/sum(beta*(1-theta[i,])*assumed.npv.symptom.Rplus.mat)
      A[i,]                <- ((alpha1[i] * beta * theta[i,])^2) * (N.symptom.plus.plus/ifelse(N.symptom.Rminus.mat[i,]==0,10^1000000000,N.symptom.Rminus.mat[i,])) *
                             assumed.npv.symptom.Rminus.mat * (1-assumed.npv.symptom.Rminus.mat)
      test                <- rep(NA, NROW(N.symptom.plus.plus))
      for(ii in 1:NROW(N.symptom.plus.plus)) {
        test[ii]          <- ifelse( (N.symptom.plus.plus[ii] - N.symptom.Rminus.mat[i,][ii])==0,
                                     0,
                                     N.symptom.plus.plus[ii] / (N.symptom.plus.plus[ii] - N.symptom.Rminus.mat[i,][ii]))
      }
      B[i,]                <- ((alpha2[i] * beta * (1-theta[i,]))^2) * (test) * assumed.npv.symptom.Rplus.mat * (1-assumed.npv.symptom.Rplus.mat);
      weight.mat[i,]      <- sqrt(A[i,]+B[i,])/sum(sqrt(A[i,]+B[i,]))
      sampsi.mat[i]       <- 1 / k.fpf * sum(sqrt(A[i,] + B[i,])) ^ 2
    }
    #Here is the final output for 1 particular combination of 
    sampsi.symptom.mat    <- ceiling(round(weight.mat,3)*ceiling(sampsi.mat))
    out.mat.sp            <- (cbind(round(weight.mat,3), sampsi.symptom.mat, apply(sampsi.symptom.mat,1,sum)))
    colnames(out.mat.sp)  <-  c(paste(colnames(N.symptom.Rplus.mat),": W",sep=""),paste(colnames(N.symptom.Rplus.mat),": N",sep=""),"Total")
    rownames(out.mat.sp)  <- rownames(N.symptom.Rplus.mat)
    
    #Adding the following information to communicate to clinicians
    se <- sqrt(vboundonlogit.fpf)
    lclLOGIT <- log(assumed.fpf/(1-assumed.fpf))-qnorm(0.975)*sqrt(vboundonlogit.fpf)
    uclLOGIT <- log(assumed.fpf/(1-assumed.fpf))+qnorm(0.975)*sqrt(vboundonlogit.fpf)
    lcl      <- (exp(lclLOGIT)/(1+exp(lclLOGIT)))
    ucl      <- (exp(uclLOGIT)/(1+exp(uclLOGIT)))
    
    specificity <- list(BETAS  = beta,
                        THETA  = theta,
                        ALPHA1 = alpha1,
                        ALPHA2 = alpha2,
                        A      = A,
                        B      = B,
                        RESULT = out.mat.sp,
                        assumed.tpf = assumed.tpf,
                        lcl.assumed.tpf = lcl,
                        ucl.assumed.tpf = ucl,
                        assumptions = list(
                          vbound.fpf        = match.call()[[9]],
                          fpf               = match.call()[[10]],
                          npv.symptom.Rplus = match.call()[[11]],
                          npv.symptom.Rminus= match.call()[[12]])
                        )
    }
    
    out         <- list(sen = sensitivity,
                        spe = specificity
                       )
    return(out)
}
# #Grab some data
# source("20-phenotypeSummary.R")
# #Run the function under various assumptions 
# #Assumption 1: (page 175) - NAIVE estimates: TPF=85% , FPF=26%, PPV=29.6%, NPV=97.5%
# senario1    <-  BeggAndGreeneSampleSizeCalculation( phenotype.names             = c("AHRQ"),
#                                                     symptom_class               = "symptom_class",
#                                                     Freq                        = "Freq",
#                                                     dat                         = sumariz,
#                                                     vbound.tpf                  = (0.05/2)^2,
#                                                     assumed.tpf                 = 0.8,
#                                                     assumed.ppv.symptom.Rplus   = list("2: All 1's"=.85,"1: All 0's"=0.0),
#                                                     assumed.ppv.symptom.Rminus  = list("1: All 0's"=0.60,"2: All 1's"=0.0),
#                                                     vbound.fpf                  = (0.05/2)^2,
#                                                     assumed.fpf                 = 0.8,
#                                                     assumed.npv.symptom.Rplus   = list("1: All 0's"=0.0, "2: All 1's"=.85),
#                                                     assumed.npv.symptom.Rminus  = list("1: All 0's"=0.60,"2: All 1's"=0.0)
#                                                     )
# #Specificity Results
# senario1$spe$RESULT
# #Sensitivity Results
# senario1$sen$RESULT
# senario2    <-  BeggAndGreeneSampleSizeCalculation(phenotype.names       = c("AHRQ"),
#                                 vbound              = (0.05/2)^2,
#                                 assumed.tpf           = 0.1,
#                                 assumed.ppv.symptom.Rplus   = list("1"=0.0, "2"=.85),
#                                 assumed.ppv.symptom.Rminus  = list("1"=0.60,"2"=0.0),
#                                 assumed.npv.symptom.Rplus   = list("1"=0.00,"2"=0.80),
#                                 assumed.npv.symptom.Rminus  = list("1"=0.30,"2"=0.0),
#                                 symptom_class       = sumariz$symptom_class,
#                                 dat                 = sumariz)
# #Specificity Results
# senario2$spe$RESULT
# #Sensitivity Results
# senario2$sen$RESULT


