#######################################################################################################
# File-Name:       BeggAndGreeneTPF and FPFestimators.R           
# Date:            7.18.2014                                
# Author:          (Nigel) Benjamin Neely (nigelneely@gmail.com)
# Purpose:         To implement estimators of TPF and FPF according to the paper "Assessment of Diagnostic 
#                  Tests When Disease Verification is Subject to Selection Bias" - Colin B. Begg and 
#                  Robert A. Greenes. Biometrics, Vol. 39, No. 1 (Mar., 1983), pp. 207-215
# Data Used:       N/A
# Packages Used:   plyr,reshape2
#######################################################################################################
# NOTE: If you are running this in the R console you must use the 'setwd' command to set the 
# working directory for the console to whereever you have saved this file prior to running.
# Otherwise you will see errors when loading data or saving figures!
#######################################################################################################
BeggAndGreeneTPFandFPFestimators <- function(symptom_class, #Variable that contains "symptom classes" mutually exclusive grouping
                                             dat,           #Data that contains phenotypes and symptom_class variable
                                             sampVarName,   #Variable that indicates sampled record
                                             testName,      #Test to be varified can be list (i.e. c("AHRQ","CMSCCW")) 
                                             goldStandard   #Adjudicated variable name
                                             ){                           
    suppressMessages(require(plyr))
    suppressMessages(require(reshape2))
    #Standardized the input that we'll feed into this program
    #Inputs to the function
    dat <- dat[order(eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))),]
    #get actual data
    sc <- eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))
    ################################################################################################################################
    #Please note the following designation for letters used throughout this program:
    #   R,       value of diagnostic test result   (i.e. Authoritative Source)
    #   D,       disease status                    (i.e. Gold Standard)
    #   V,       selection variable               
    #     V+:    case selected for definitive disease verification
    #     V-:    case NOT selected for definitive disease verification
    #   X,       concomitant information (vector)
    #Using these variables we will determine the sample size needed to determine certain sensitivity/specificity defintions
    ################################################################################################################################
    #Gather some inital information about incoming information and instantiate the N.symptom dataset
    #This data set will be used to generate all information regarding N++, N+1, n+1, information as seen in the theoretical statistical appendix
    n.test                <- length(testName)
    n.symptoms            <- length(unique(eval(parse(text = paste("dat$", deparse(symptom_class), sep="")))))
    sumariz               <- data.frame(table(dat[,c(testName,symptom_class,sampVarName,goldStandard)],useNA="ifany"))
    #Not elegant, but works. Loop through to generate a data set that will contain information that will allow us to calculate sample size
    #based on two statistics: (1) Sensisitivity and/or (2)Specificity
    for (i in 1:n.test) {
      if (i==1) {
        N.symptom         <- data.frame(symptom_class=numeric(),
                                        R=numeric(),
                                        V=numeric(),
                                        D=numeric(),
                                        total=numeric(),
                                        authSource=character())
      }
      #To calculate estimates of interest, we need to define several quantities of interest
      temp                <- data.frame( ddply(sumariz,.(symptom_class,sumariz[,testName[i]],sumariz[,sampVarName],sumariz[,goldStandard]),
                                               summarise,total=sum(Freq)) )
      colnames(temp)      <- c("symptom_class","R","V","D","total")
      final               <- temp
      final$total         <- ifelse(is.na(final$total),0,final$total)
      final$authSource    <- rep(testName[i],length(final$total))
      N.symptom           <- data.frame(rbind(N.symptom,final))
      rm(temp,final)
    }
    #Create a function to create matrix/vectors needed 
    createMat <- function(sub) {
      temp.df           <- dcast(melt(sub,id.vars=c("symptom_class","authSource"),measure.vars="total"),authSource~symptom_class,sum)
      rownames(temp.df) <- temp.df$authSource
      temp.mt           <- data.matrix(temp.df[2:ncol(temp.df)],rownames.force=TRUE) 
      return(temp.mt)
    }
    N.symptom.Rplus.Vplus.mat           <- createMat(subset(N.symptom,(R==1 & V==1))) 
    N.symptom.Rplus.mat                 <- createMat(subset(N.symptom,(R==1))) 
    N.symptom.Dplus.Rplus.Vplus.mat     <- createMat(subset(N.symptom,(R==1 & V==1 & D==1))) 
    N.symptom.Dminus.Rplus.Vplus.mat    <- createMat(subset(N.symptom,(R==1 & V==1 & D==0))) 
    N.symptom.Rminus.Vplus.mat          <- createMat(subset(N.symptom,(R==0 & V==1)))
    N.symptom.Dplus.Rminus.Vplus.mat    <- createMat(subset(N.symptom,(R==0 & V==1 & D==1))) 
    N.symptom.Dminus.Rminus.Vplus.mat   <- createMat(subset(N.symptom,(R==0 & V==1 & D==0)))
    N.symptom.Rminus.mat                <- createMat(subset(N.symptom,(R==0))) 
    N.symptom.plus.plus.atomic          <- as.numeric(by(sumariz$Freq,sumariz[,symptom_class],sum))
    
    ####################################################################
    #TPF Calculation
    ####################################################################
    beta                                <- N.symptom.plus.plus.atomic/sum(N.symptom.plus.plus.atomic)
    theta.Rplus                         <- array(NA, c(n.test, n.symptoms))
    phi.Dplus                           <- array(NA, c(n.test, n.symptoms))
    psi.Dplus                           <- array(NA, c(n.test, n.symptoms))
    var.phi.Dplus                       <- array(NA, c(n.test, n.symptoms))
    var.psi.Dplus                       <- array(NA, c(n.test, n.symptoms))
    alpha1.Dplus                        <- array(NA, c(n.test,1))
    alpha2.Dplus                        <- array(NA, c(n.test,1))
    log.minuend.Dplus                   <- array(NA, c(n.test,1))
    log.subtrahend.Dplus                <- array(NA, c(n.test,1))
    logit.TPF                           <- array(NA, c(n.test,1))
    TPF                                 <- array(NA, c(n.test,1))
    augend.Dplus                        <- array(NA, c(n.test,1))
    addend.Dplus                        <- array(NA, c(n.test,1))
    var.logit.TPF                       <- array(NA, c(n.test,1))
    var.TPF                             <- array(NA, c(n.test,1))
    #For each Authoritative Source, let's grab the information that we're interested in...
    for(i in 1:n.test) {
        theta.Rplus[i,]         <- N.symptom.Rplus.mat[i,]/N.symptom.plus.plus.atomic
        phi.Dplus[i,]           <- N.symptom.Dplus.Rplus.Vplus.mat[i,] / N.symptom.Rplus.Vplus.mat[i,]
        psi.Dplus[i,]           <- N.symptom.Dplus.Rminus.Vplus.mat[i,] / N.symptom.Rminus.Vplus.mat[i,]
        alpha1.Dplus[i,]        <- 1/(sum(beta*theta.Rplus[i,]*phi.Dplus[i,]))
        alpha2.Dplus[i]         <- 1/(sum(beta*(1-theta.Rplus[i,])*psi.Dplus[i,]))
        log.minuend.Dplus[i,]   <- log(1/alpha1.Dplus[i,]) #Alpha is actually the inverse, so we need to switch it back
        log.subtrahend.Dplus[i,]<- log(1/alpha2.Dplus[i,]) #Alpha is actually the inverse, so we need to switch it back
        logit.TPF[i,]           <- log.minuend.Dplus[i,]-log.subtrahend.Dplus[i,]
        TPF[i,]                 <- (exp(logit.TPF[i,])/(1+exp(logit.TPF[i,])))
        var.phi.Dplus[i,]       <- (phi.Dplus[i,]*(1-phi.Dplus[i,]))/(N.symptom.Rplus.mat[i,])
        var.psi.Dplus[i,]       <- (psi.Dplus[i,]*(1-psi.Dplus[i,]))/(N.symptom.Rminus.mat[i,])
        augend.Dplus[i,]        <- sum((alpha1.Dplus[i,]*beta*theta.Rplus[i,])^(2)*var.phi.Dplus[i,])
        addend.Dplus[i,]        <- sum((alpha2.Dplus*beta*(1-theta.Rplus[i,]))^(2)*var.psi.Dplus[i,])
        var.logit.TPF[i,]       <- augend.Dplus[i,] + addend.Dplus[i,]
        var.TPF[i,]             <- var.logit.TPF[i,]/((1/TPF[i,])+(1/(1-TPF[i,])))^2 #Using Casella/Berger pg.242 to get back to the quantitiy of interest
    }
    rownames(TPF)     <- rownames(N.symptom.Rplus.Vplus.mat)
    rownames(var.TPF) <- rownames(N.symptom.Rplus.Vplus.mat)
    ####################################################################
    #FPF Calculation
    ####################################################################
    theta.Rplus                         <- array(NA, c(n.test, n.symptoms))
    phi.Dminus                          <- array(NA, c(n.test, n.symptoms))
    psi.Dminus                          <- array(NA, c(n.test, n.symptoms))
    var.phi.Dminus                      <- array(NA, c(n.test, n.symptoms))
    var.psi.Dminus                      <- array(NA, c(n.test, n.symptoms))
    alpha1.Dminus                       <- array(NA, c(n.test,1))
    alpha2.Dminus                       <- array(NA, c(n.test,1))
    log.minuend.Dminus                  <- array(NA, c(n.test,1))
    log.subtrahend.Dminus               <- array(NA, c(n.test,1))
    logit.FPF                           <- array(NA, c(n.test,1))
    FPF                                 <- array(NA, c(n.test,1))
    augend.Dminus                       <- array(NA, c(n.test,1))
    addend.Dminus                       <- array(NA, c(n.test,1))
    var.logit.FPF                       <- array(NA, c(n.test,1))
    var.FPF                             <- array(NA, c(n.test,1))
    #For each Authoritative Source, let's grab the information that we're interested in...
    for(i in 1:n.test) {
      theta.Rplus[i,]           <- N.symptom.Rplus.mat[i,]/N.symptom.plus.plus.atomic
      phi.Dminus[i,]            <- N.symptom.Dminus.Rplus.Vplus.mat[i,] / N.symptom.Rplus.Vplus.mat[i,]
      psi.Dminus[i,]            <- N.symptom.Dminus.Rminus.Vplus.mat[i,] / N.symptom.Rminus.Vplus.mat[i,]
      alpha1.Dminus[i,]         <- 1/sum(beta*theta.Rplus[i,]*phi.Dminus[i,])
      alpha2.Dminus[i]          <- 1/sum(beta*(1-theta.Rplus[i,])*psi.Dminus[i,])
      log.minuend.Dminus[i,]    <- log(1/alpha1.Dminus[i,]) #Alpha is actually the inverse, so we need to switch it back
      log.subtrahend.Dminus[i,] <- log(1/alpha2.Dminus[i,]) #Alpha is actually the inverse, so we need to switch it back
      logit.FPF[i,]             <- log.minuend.Dminus[i,]-log.subtrahend.Dminus[i,]
      FPF[i,]                   <- (exp(logit.FPF[i,])/(1+exp(logit.FPF[i,])))
      var.phi.Dminus[i,]        <- (phi.Dminus[i,]*(1-phi.Dminus[i,]))/(N.symptom.Rplus.mat[i,])
      var.psi.Dminus[i,]        <- (psi.Dminus[i,]*(1-psi.Dminus[i,]))/(N.symptom.Rminus.mat[i,])
      augend.Dminus[i,]         <- sum((alpha1.Dminus[i,]*beta*theta.Rplus[i,])^(2)*var.phi.Dminus[i,])
      addend.Dminus[i,]         <- sum((alpha2.Dminus*beta*(1-theta.Rplus[i,]))^(2)*var.psi.Dminus[i,])
      var.logit.FPF[i,]         <- augend.Dminus[i,] + addend.Dminus[i,]
      var.FPF[i,]               <- var.logit.FPF[i,]/((1/FPF[i,])+(1/(1-FPF[i,])))^2 #Using Casella/Berger pg.242 to get back to the quantitiy of interest
    }
    rownames(FPF)       <- rownames(N.symptom.Rplus.Vplus.mat)
    rownames(var.FPF)   <- rownames(N.symptom.Rplus.Vplus.mat)
    
    results             <-list(beta=beta,
                               TPF=list(
                                                phi.Dplus=phi.Dplus,
                                                psi.Dplus=psi.Dplus,
                                                alpha1.Dplus=alpha1.Dplus,
                                                alpha2.Dplus=alpha2.Dplus,
                                                log.minuend.Dplus=log.minuend.Dplus,
                                                log.subtrahend.Dplus=log.subtrahend.Dplus,
                                                logit.TPF=logit.TPF,
                                                TPF=TPF,
                                                var.TPF=var.TPF),
                               FPF=list(
                                                phi.Dminus=phi.Dminus,
                                                psi.Dminus=psi.Dminus,
                                                alpha1.Dminus=alpha1.Dminus,
                                                alpha2.Dminus=alpha2.Dminus,
                                                log.minuend.Dminus=log.minuend.Dminus,
                                                log.subtrahend.Dminus=log.subtrahend.Dminus,
                                                logit.FPF=logit.FPF,
                                                FPF=FPF,
                                                var.FPF=var.FPF))
    return(results)
   
}
# source("10-GenerateData_7.2.R")
# simdat$symptom_class  <- factor(simdat$symptom_class,
#                                 levels=c(0,1),
#                                 labels=c("1: All 0's","2: All 1's"))
# BeggAndGreeneTPFandFPFestimators(symptom_class="symptom_class", #Variable that contains "symptom classes" mutually exclusive grouping
#                                  dat=simdat,              #Data that contains phenotypes and symptom_class variable
#                                  sampVarName="selected",  #Variable that indicates sampled record
#                                  testName=c("AHRQ"),         #Variable that indicates test to be varified (R)
#                                  goldStandard="Death"     #Adjudicated variable name
#                                  )


